from .api import API
from .grpc import VegaTradingClient as grpcTradingClient
from .rest import VegaTradingClient as restTradingClient


class VegaTradingClient(object):

    def __init__(
        self,
        url: str
    ) -> None:
        if any(url.startswith(prefix) for prefix in ["https://", "http://"]):
            self._restImpl = restTradingClient(url)
            self._api = API.REST
        else:
            self._grpcImpl = grpcTradingClient(url)
            self._api = API.GRPC

    def __getattr__(self, funcname):
        if self._api == API.GRPC:
            return getattr(self._grpcImpl, funcname)
        elif self._api == API.REST:
            return getattr(self._restImpl, funcname)
        else:
            raise Exception("API not implemented")



import base64
from typing import Any, Dict, Optional
from google.protobuf.pyext._message import (
    RepeatedScalarContainer,
    ScalarMapContainer
)


def unconvert_complextype(j, r, f):
    v = unconvert_obj(getattr(r, f.json_name))
    if v is not None:
        j[f.json_name] = v


def unconvert_field(j, r, f):
    if f.type == 11:  # object
        unconvert_complextype(j, r, f)
        return

    v = getattr(r, f.json_name)
    if isinstance(v, RepeatedScalarContainer):
        j[f.json_name] = []
        while True:
            try:
                j[f.json_name].append(v.pop())
            except IndexError:
                break
        return

    if f.type == 12:  # bytes
        j[f.json_name] = base64.b64encode(v).decode("ascii")
        return

    j[f.json_name] = v


def unconvert_obj(r) -> Optional[Dict[str, Any]]:
    """
    Convert a gRPC object into a dict.
    """
    if isinstance(r, ScalarMapContainer):
        return {k: v for k, v in r.items()}

    if not hasattr(r, "DESCRIPTOR"):
        print("No DESCRIPTOR for {} {}".format(type(r), r))
        return None

    j: Dict[str, Any] = {}
    for f in r.DESCRIPTOR.fields:
        unconvert_field(j, r, f)
    return j



import requests
from google.protobuf.reflection import GeneratedProtocolMessageType

from ..grpc import api

from .convert import convert_obj, errStr
from .unconvert import unconvert_obj


class VegaTradingClient(object):
    """
    The Vega Trading Client talks to a back-end node.
    """

    def __init__(
        self,
        url: str
    ) -> None:
        if url is None:
            raise Exception("Missing node URL")
        self.url = url

        self._httpsession = requests.Session()

    def NotifyTraderAccount(
        self,
        req: GeneratedProtocolMessageType
    ) -> GeneratedProtocolMessageType:
        url = "{}/fountain".format(self.url)
        restreq = unconvert_obj(req)
        r = self._httpsession.post(url, json=restreq)
        if r.status_code != 200:
            raise Exception(errStr(r))
        return convert_obj(r.json(), api.trading.NotifyTraderAccountResponse())

    def PrepareSubmitOrder(
        self,
        req: GeneratedProtocolMessageType
    ) -> GeneratedProtocolMessageType:
        url = "{}/orders/prepare".format(self.url)
        restreq = unconvert_obj(req)
        r = self._httpsession.post(url, json=restreq)
        if r.status_code != 200:
            raise Exception(errStr(r))
        return convert_obj(r.json(), api.trading.PrepareSubmitOrderResponse())

    # PrepareCancelOrder,CancelOrderRequest,PrepareCancelOrderResponse

    # PrepareAmendOrder,AmendOrderRequest,PrepareAmendOrderResponse

    # Withdraw,WithdrawRequest,WithdrawResponse

    def SubmitTransaction(
        self,
        req: GeneratedProtocolMessageType
    ) -> GeneratedProtocolMessageType:
        url = "{}/transaction".format(self.url)
        restreq = unconvert_obj(req)
        r = self._httpsession.post(url, json=restreq)
        if r.status_code != 200:
            raise Exception(errStr(r))
        return convert_obj(r.json(), api.trading.SubmitTransactionResponse())
